from email.MIMEText import MIMEText
from email.MIMEMultipart import MIMEMultipart
import smtplib

try:
    import docutils
    import docutils.core
except ImportError:
    docutils = None

from quixote import get_request

import storage

from form import *

widget_classes = {
    'string': StringWidget,
    'text': TextWidget,
    'bool': CheckboxWidget,
    'item': SingleSelectWidget,
}


class FormDef(storage.Storable):
    key = 'id'
    names = 'formdefs'
    receiver = None
    roles = None

    def __init__(self):
        self.id = ''
        self.name = ''
        self.fields = []
        self.receiver = ''
        self.emailrcpt = ''
        self.roles = []

    def create_form(self):  # XXX: merge create_form and create_view_form
        form = Form(enctype = "multipart/form-data")
        for i, field in enumerate(self.fields):
            if field['type'] == 'title':
                form.widgets.append(HtmlWidget(htmltext("<h3>%s</h3>" % field['name'])))
                continue
            if field['type'] == 'subtitle':
                form.widgets.append(HtmlWidget(htmltext("<h4>%s</h4>" % field['name'])))
                continue
            kwargs = {'required': field.get('required', True)}
            if field['type'] == 'item':
                kwargs['options'] = field.get('items', ['---'])
            form.add(widget_classes[field['type']], "f%d" % i, title = field['name'],
                    **kwargs)
        return form

    def create_view_form(self, dict = {}):
        form = Form(enctype = "multipart/form-data")
        for i, field in enumerate(self.fields):
            kwargs = {}
            if field['type'] == 'title':
                form.widgets.append(HtmlWidget(htmltext("<h3>%s</h3>" % field['name'])))
                continue
            if field['type'] == 'subtitle':
                form.widgets.append(HtmlWidget(htmltext("<h4>%s</h4>" % field['name'])))
                continue
            if field['type'] == 'item':
                kwargs['options'] = field.get('items', ['---'])
            form.add(widget_classes[field['type']], "f%d" % i, title = field['name'],
                    readonly = "readonly", value = dict.get(field['name'], ''), **kwargs)
        return form

    def get_data(self, form):
        d = {}
        for i, field in enumerate(self.fields):
            if field['type'] in ('title', 'subtitle'):
                continue
            d[field['name']] = form.get_widget('f%d' % i).parse()
        return d

    def notify_new(self, formdata):
        if not self.emailrcpt:
            return
        url = '%s/%s/status' % (get_request().get_url(1), formdata.id)
        mail_body = _("""Hi,

A new form has been submitted on the website, you can consult it with this
link: %(url)s
""") % {'url': url}
        try:
            htmlmail = docutils.core.publish_string(mail_body, writer_name="html")
        except IOError:
            htmlmail = None

        msg = MIMEText(mail_body, _charset = 'iso-8859-15')
        msg['Subject'] = _("A new form has been submitted")
        msg['To'] = self.emailrcpt
        msg['From'] = 'WCS <noreply@entrouvert.com>'
        msg['X-Mailer'] = 'w.c.s.'

        s = smtplib.SMTP()
        s.connect()
        s.sendmail('noreply@entrouvert.com', [msg['To']], msg.as_string())
        s.close()

