import os
import cPickle

class Storage:
    pass


class Storable:
    def get_table_name(self):
        if hasattr(self, 'names'):
            return self.names
        return self.__class__.__name__.lower()

def fixkey(k):
    # insure key can be inserted in filesystem
    if not k: return k
    return str(k).replace('/', '-')

class FilesStorage:
    BASEDIR = None # must be initialized

    def __init__(self):
        if self.BASEDIR and not os.path.exists(self.BASEDIR):
            os.mkdir(self.BASEDIR)

    def keys(self, obname):
        dirname = os.path.join(self.BASEDIR, str(obname))
        if not os.path.exists(dirname):
            return []
        return os.listdir(dirname)

    def store(self, object):
        obname = object.get_table_name()
        dirname = os.path.join(self.BASEDIR, obname)
        if not os.path.exists(dirname):
            os.mkdir(dirname)
        if not getattr(object, object.key):
            keys = self.keys(obname)
            key = len(keys) + 1
            while str(key) in keys:
                key += 1
            key = str(key)
            setattr(object, object.key, key)
        s = cPickle.dumps(object)
        file(os.path.join(dirname, fixkey(getattr(object, object.key))), 'w').write(s)

    def retrieve(self, obname, key):
        if key is None:
            raise KeyError()
        dirname = os.path.join(self.BASEDIR, str(obname))
        try:
            return cPickle.load(open(os.path.join(dirname, fixkey(key))))
        except IOError:
            raise KeyError()

    def values(self, obname):
        return [self.retrieve(obname, k) for k in self.keys(obname)]

    def items(self, obname):
        return [(k, self.retrieve(obname, k)) for k in self.keys(obname)]

    def remove(self, object = None):
        obname = object.get_table_name()
        return self.remove_id(obname, fixkey(getattr(object, object.key)))

    def remove_id(self, obname, key):
        dirname = os.path.join(self.BASEDIR, str(obname))
        os.unlink(os.path.join(dirname, fixkey(key)))

    def remove_all(self, obname):
        dirname = os.path.join(self.BASEDIR, str(obname))
        if not os.path.exists(dirname):
            return
        for k in os.listdir(dirname):
            os.unlink(os.path.join(dirname, k))
        os.rmdir(dirname)
        

def get_storage():
    return FilesStorage()

